// ChartComp.java
//
// Copyright (C) 1998 Henrik Bjorkman.
//
// This is class handles the map component in the chart window.
//
//
//
// History
//
// 1999-06-20
// Moved this class to its own file from ChartWindow.
// Henrik Bjorkman
//
// 1999-08-13
// The blue trace line is now rewritten when window
// is rewritten.
// Henrik
//
// 1999-08-26
// Don't save trace point unless we moved on pixel.
// Henrik
//
// 1999-08-30
// Added a move function and some other things
// needed by ShowTrace.
// Henrik
//
// 1999-09-13
// Adding functions to make it possible to move around
// in the maps by clicking with the mouse.
// Henrik
//
// 1999-09-19
// Doing setVisible to hide this while changing chart and
// to show it again after change. Perhaps this solved the
// problem with image not getting updated compleatly.
// Tried to remove the flush of old images but then I
// got out of memory after running the program a while.
// Henrik
//
// 2000-06-04
// Using the trace flag a little different to allow viewing of
// old trace files.
// Henrik
//
// 2000-06-13
// Changed package name to chartplotter_package.
// Henrik

package gps_monitor.gui;

import gps_monitor.util.*;


import java.awt.*;
import java.awt.image.*;	//Rikard
import java.awt.event.*;
import java.io.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;


public class ChartComp extends JPanel{

  final int redCircle=20;  // The radius of the circle in the position mark
  final int greenCross=20; // The size of the cross in the position mark
  Image img=null;          // The chart being show on screen
  Image oldImg=null;       // A previous chart
  boolean valid=false;     // True if position info has timed out
  Point pred=null;         // Predicted position
  Point pos=null;          // Position of the mark
  int precision;           // The estimated precision on the chart in pixels
  int scale;               // The number of pixels per 100 (or 50) meters.
  boolean trace;           // If true we plot a trace line.
  int thickness;           // How thick the red circle shall be
  Point[] tracePoints=new Point[2000];  // To remember where we have plotted a blue line.
  int traceIndex;          // Where in the array above we shall store next point.
  Point lastPos=null;      // Used so we don't save same point twice.
  Frame parent=null;       // The frame in which this component was created.
  final int BORDER=5;      // It seems some extra margin is needed otherwise image is cut */
  Point mousePos=null;     // Position for mouse click
  Object objectToNotify=null;
  int heading = 180;

	private Calling calling;
	int range = 0;

  public void setNotify(Object objectToNotify)
  {
    this.objectToNotify=objectToNotify;
  }



  static void error(String s)
  {
    System.err.println("ChartComp: " + s);
  }



	public Point getLocation(){
		return pos;
	}



  // Load an image file.
  // name: Name of image file to load. Usually a .jpg file.
  // scale: the radius for the black ring. Usually 100 meters.
  // precision: the radius for the yellow ring. Usually not used.
  synchronized public void loadChart(String name, int scale, int precision)
  {
    this.setVisible(false);

    //if ((oldImg!=null) && (oldImg!=img)) {oldImg.flush();}
    //oldImg=img;
    if (img!=null) {img.flush();}

    img = getToolkit().getImage("files/maps/"+name);

    for (int i=0;i<tracePoints.length;i++)
    {
      tracePoints[i]=null;
    }
    lastPos=null;


    this.setSize(this.getPreferredSize());
    this.scale=scale;
    this.precision=precision;
    //if (this.precision==redCircle) {this.precision++;}



    this.setVisible(true);
  }



  synchronized public void loadChart(String name)
  {
    loadChart(name,-1,-1);
  }



  public ChartComp(Frame parent, boolean trace, int thickness)
  {
    this.parent=parent;
    this.trace=trace;
    this.thickness=thickness;

    this.enableEvents(AWTEvent.MOUSE_EVENT_MASK);
  }





  synchronized public Dimension getPreferredSize()
  {
    if (img!=null)
    {

      // Wait until the image is loaded so we know the size
      int i=0;
      while ((i<30) && (img.getWidth(this)<0))
      {
        try {this.wait(100);} catch (InterruptedException e) {;}
        i++;
      }



      return new Dimension(img.getWidth(this)+BORDER, img.getHeight(this)+BORDER);
    }
    else
    {
      return new Dimension(120,100);
    }
  }



	public void startCall(){
		calling = new Calling();
		calling.start();
	}

	public void stopCall(){
		calling.stop();
		range = 0;
		paint();
	}

	private class Calling extends Thread{

		public void run(){

			while(true){

				try{ Thread.sleep(750); }catch(Exception e){}

				range += 8;
				if( range > 48 )
					range = 8;
				paint();

			}

		}

	}



  synchronized void drawCross(Graphics g)
  {
    if (pos!=null)
    {

		Image img2 = getToolkit().getImage("files/users/user0_"+heading+".gif");
		g.drawImage(img2, pos.x, pos.y, this);

		g.setColor(Color.red);
		//for(int j=8; j<=range; j+=8){
		if( range > 8 ){
			g.drawOval(pos.x-range+15,pos.y-range+15,range*2,range*2);
			Color c = Color.red;
			g.setColor(new Color(c.getRed(), c.getGreen(), c.getBlue(), 255-100 ));
			g.drawOval(pos.x-(range-8)+15,pos.y-(range-8)+15,(range-8)*2,(range-8)*2);
			c = Color.red;
			g.setColor(new Color(c.getRed(), c.getGreen(), c.getBlue(), 255-150 ));
			g.drawOval(pos.x-(range-16)+15,pos.y-(range-16)+15,(range-16)*2,(range-16)*2);
		}else if( range > 0 ){
			g.drawOval(pos.x-range+15,pos.y-range+15,range*2,range*2);
			Color c = Color.red;
			g.setColor(new Color(c.getRed(), c.getGreen(), c.getBlue(), 255-100 ));
			g.drawOval(pos.x-(range-8)+15,pos.y-(range-8)+15,(range-8)*2,(range-8)*2);
		}
		//}


    /*if( heading > -1.0 ){


		double x = Math.cos((Math.PI*(heading/180.0))) * 40;
		double y = Math.sin((Math.PI*(heading/180.0))) * 40;

		double arrow1 = heading + 25;
		double arrow2 = heading - 25;

		double x1 = Math.cos((Math.PI*(arrow1/180.0))) * 12;
		double y1 = Math.sin((Math.PI*(arrow1/180.0))) * 12;

		double x2 = Math.cos((Math.PI*(arrow2/180.0))) * 12;
		double y2 = Math.sin((Math.PI*(arrow2/180.0))) * 12;

		g.setColor(Color.red);
		g.drawLine(pos.x,pos.y,(int)(x+pos.x),(int)(pos.y-y));
		g.drawLine((int)(x+pos.x),(int)(pos.y-y),(int)((x+pos.x)-x1),(int)((pos.y-y)+y1));
		g.drawLine((int)(x+pos.x),(int)(pos.y-y),(int)((x+pos.x)-x2),(int)((pos.y-y)+y2));

	}

      g.setXORMode(Color.white);

      g.setColor(Color.green);
      if (valid)
      {
        g.drawLine(pos.x-greenCross,pos.y,pos.x+greenCross,pos.y);
        g.drawLine(pos.x,pos.y-greenCross,pos.x,pos.y+greenCross);
      }

      g.setColor(Color.red);
      for (int i=0;i<thickness;i++)
      {
        g.drawOval(pos.x-(redCircle+i),pos.y-(redCircle+i),2*(redCircle+i),2*(redCircle+i));
      }

      // precision ring
      if ((precision>0) && (precision<200))
      {
        g.setColor(Color.yellow);
        g.drawOval(pos.x-precision,pos.y-precision,2*precision,2*precision);
      }

      // scale ring
      if ((scale>0) && (scale<200))
      {
        g.setColor(Color.black);
        g.drawOval(pos.x-scale,pos.y-scale,2*scale,2*scale);
      }

      if ((valid) && (pred!=null))
      {
        g.setColor(Color.black);
        g.drawLine(pos.x,pos.y,pred.x,pred.y);
      }*/
    }
  }



	///
	//
	// This draws a trace line!!
	//
	//
  public synchronized void plottPos(Point pos, Point pred){

    Graphics g=getGraphics();

    drawCross(g);

    if ((this.pos!=null) && (pos!=null) && trace)
    {
      g.setPaintMode();
      g.setColor(Color.blue);
      //g.drawLine(pos.x,pos.y,this.pos.x,this.pos.y);
    }

    this.pos=pos;
    this.pred=pred;
    valid=true;
    drawCross(g);



    g.dispose();
  }




  public synchronized void updatePos(Point pos, Point pred, double nheading)
  {

	  int temp = (int)(nheading/45);
	  this.heading = temp*45;
    plottPos(pos, pred);
    if (trace) {tracePos(pos);}
  }



  public synchronized void tracePos(Point pos)
  {
    if ((lastPos==null) || (pos==null) ||
     (lastPos.x!=pos.x) || (lastPos.y!=pos.y))
    {
      tracePoints[traceIndex]=pos;
      if (++traceIndex>=tracePoints.length) {traceIndex=0;}
      tracePoints[traceIndex]=null;
      lastPos=pos;
    }
  }




  	synchronized public void movePoint(int x, int y){

    	Graphics g=getGraphics();
    	if (pos!=null) {drawCross(g);}
    	pos.x+=x;
    	pos.y+=y;
    	drawCross(g);
    	g.dispose();
  	}




  	public synchronized void timeOut(){

    	Graphics g=getGraphics();
    	drawCross(g);
    	valid=false;
    	drawCross(g);
    	g.dispose();
  	}




  	public void update(Graphics g){

    	paint(g);
  	}



  	public synchronized void paint(){

    	Graphics g=getGraphics();
    	paint(g);
    	g.dispose();
  	}



  public synchronized void paint(Graphics g)
  {
    if (img!=null) {g.drawImage(img, 0, 0, this);}

    g.setPaintMode();
    g.setColor(Color.blue);

    //if (trace)
    {
      Point p=null;
      int j=traceIndex;
      for (int i=0;i<tracePoints.length;i++)
      {
        if (++j>=tracePoints.length) {j=0;}
        if (tracePoints[j]!=null)
        {
          if (p!=null)
          {
            //g.drawLine(p.x,p.y,tracePoints[j].x,tracePoints[j].y);
          }
        }
        p=tracePoints[j];
      }
    }

    drawCross(g);
    g.dispose();
  }






  synchronized public void processMouseEvent(MouseEvent e)
  {
    if (e.getID()==MouseEvent.MOUSE_PRESSED)
    {
      //if (point!=null) {drawCross(point);}
      //point=new Point(e.getX(),e.getY());
      //drawCross(point);
      mousePos=new Point(e.getX(),e.getY());


      /*if (objectToNotify!=null)
      {
        synchronized(objectToNotify) {objectToNotify.notify();}
      }*/
    }
    else
    {
      super.processMouseEvent(e);
    }
  }


  synchronized public Point getMousePos()
  {
    Point p=mousePos;
    mousePos=null;
    return(p);
  }

}

