
/**
 * This class has been created to help provide an abstract representation
 * of a geospatial (i.e. latitude, longitude and altitude) coordinate.
 * It is not intended to oppose the Java3D concept,
 * and there are methods within LocationConverter in this package that may
 * help conversion between geospatial and cartesian forms.
 *
 * I felt it necessary to write this class as opposed to simply use the
 * Java3D package.  This is because I did not want users to have to install
 * the non-standard package for my use of one class, this seemed excessive.
 * Also a geosptial coordinate is not conceptually compatible with the Java3D
 * coordinate, which is of cartesian form.
 *
 * The altitude values are assumed to all be in metres above sea-level
 * according to the WGS84 ellipsoid.
 *
 * @see locationUtils.LatOrLongitude
 * @author Richard Caudle
 * @version 1.0.0
 */

package gps_monitor.util;

public class GlobalCoordinate
{
	/**
	* Declare the variables that will be used to store the values
	* needed by this class.
	*/
	LatOrLongitude latitude;
	LatOrLongitude longitude;
	double altitude;


	/**
	 * This constructor creates a new GlobalCoordinate.
	 *
	 * @param lat The latitude of the new coordinate
	 * @param lon The longitude of the new coordinate
	 * @param alt The altitude of the new coordinate
	 */
	public GlobalCoordinate(LatOrLongitude lat, LatOrLongitude lon, double alt)
	{
		latitude = lat;
		longitude = lon;
		altitude = alt;
	}

	/**
	 * This method returns the latitude of this coordinate
	 *
	 * @return The latitude of the coordinate
	 */
	public LatOrLongitude getLatitude()
	{
		return latitude;
	}

	/**
	 * This method returns the longitude of this coordinate
	 *
	 * @return The longitude of the coordinate
	 */
	public LatOrLongitude getLongitude()
	{
		return longitude;
	}

	/**
	 * This method returns the altitude of this coordinate
	 *
	 * @return The altitude of the coordinate
	 */
	public double getAltitude()
	{
		return altitude;
	}

	/**
	 * This method allows us to set the latitude of the coordinate
	 *
	 * @param The latitude the coordinate should be set to
	 */
	public void setLatitude(LatOrLongitude lat)
	{
		latitude = lat;
	}

	/**
	 * This method allows us to set the longitude of the coordinate
	 *
	 * @param The longitude the coordinate should be set to
	 */
	public void setLongitude(LatOrLongitude lon)
	{
		longitude = lon;
	}

	/**
	 * This method allows us to set the altitude of the coordinate
	 *
	 * @param The altitude the coordinate should be set to
	 */
	public void setAltitude(double alt)
	{
		altitude = alt;
	}

	/**
	 * A method to test whether two GlobalCoordinates objects are equal.
	 * The method compares this object to the coordinate, by comparing
	 * the three common attributes (latitude, longitude, and altitude).
	 *
	 * @param obj The object which this will compared to.
	 * @return Whether the two objects are equal
	 */
	public boolean equals(GlobalCoordinate obj)
	{
		if (obj != null)
		  return (getLatitude().equals(obj.getLatitude())) &&
			   (getLongitude().equals(obj.getLongitude())) &&
			   (getAltitude() == obj.getAltitude());
		else
		  return false;
	}
}
